 package graaf;

 import java.util.*; // Vector
 import java.io.*;

/*********************************************************
 * Graafiga tegelemine.
 *@author Jri Kiho
 *@version 19.04.2009
*/ 
 public class Graaf
  {
/**
 * Graafi tippude koht.
*/
  protected Vector <Tipp> tipud = new Vector <Tipp> ();
/**
 * Graafi kaarte koht.
*/
  protected Vector <Kaar> kaared = new Vector <Kaar> ();

/*****************************
 * Piilumeetod.
*/
 public Vector<Tipp> tipud(){
        return tipud;
 }

/*****************************
 * Piilumeetod.
*/
  public Vector<Kaar> kaared(){
        return kaared;
 }

/******************************************************
 * Koopia tegemine. 
 *@return Selle graafi koopia.
*/
	public Graaf koopia(){
		Graaf g = new Graaf();
		Tipp[] uus = new Tipp[n()]; // tippude koopiate koht
		for(int i = 0; i < n(); i++){
			uus[i] = new Tipp(tipp(i).nimi());
			g.tipud().add(uus[i]);
		}
		for(int i = 0; i < m(); i++){
			Kaar k = kaar(i);
			Tipp alg = k.algus();
			Tipp lpp = k.lpp();
			Tipp uusAlg = uus[indeks(alg)];
			Tipp uusLpp = uus[indeks(lpp)];
			g.kaared.add(new Kaar(uusAlg, uusLpp, k.nimi()));
		}
		return g;
	}

/******************************************************
 * Tipu lisamine. 
 * Antud tipp lisatakse selle graafi tippude hulka.
 *@param t lisatav tipp
 *@return Lisatud tipp.
*/
 public Tipp lisada(Tipp t){
        tipud.add(t);
        return t;
 }

/******************************************************
 * Kaare lisamine. 
 * Antud kaar lisatakse selle graafi kaarte hulka. 
 *@param k lisatav kaar 
 *@return Lisatud kaar.
 *@throws IllegalArgumentException kui mni kaare otstippudest ei kuulu selle graafi tippude hulka, siis
*/
 public Kaar lisada(Kaar k){
	if(!tipud.contains(k.algus()) || !tipud.contains(k.lpp()))
		throw new IllegalArgumentException(); 
        kaared.add(k);
        return k;
 }

/******************************************************
 * Tippude arv. 
 *@return Selle graafi tippude arv.
*/
 public int n(){
        return tipud.size();
 }

/******************************************************
 * Kaarte arv. 
 *@return Selle graafi kaarte arv.
*/
 public int m(){
        return kaared.size();
 }

/******************************************************
 * Jrjekordne tipp. 
 *@param i tipu indeks 
 *@return Selle graafi <i>i</i>-s tipp.
 *@throws IndexOutOfBoundsException kui indeks vale
*/
 public Tipp tipp(int i){
        return tipud.elementAt(i);
 }

/******************************************************
 * Jrjekordne kaar. 
 *@param i kaare indeks 
 *@return Selle graafi <i>i</i>-s kaar.
 *@throws IndexOutOfBoundsException kui indeks vale
*/
 public Kaar kaar(int i){
        return kaared.elementAt(i);
 }

/******************************************************
 * Kaar kahe tipu vahel. 
 *@param t vimaliku kaare algustipp
 *@param u vimaliku kaare lpptipp 
 *@return Selle graafi kaar <i>t-->u</i>, vi <code>null</code> (kui kaart pole).
*/
 public Kaar kaar(Tipp t, Tipp u){
    for(Kaar k: kaared)
       if (k.algus() == t && k.lpp() == u)
              return k;
    return null;
 }

/******************************************************
 * Tipu indeks. 
 *@param t antud tipp
 *@return Tipu <code>t</code> jrjekorranumber selle graafi tippude vektoris (vi -1).
*/
 public int indeks(Tipp t){
        return tipud.indexOf(t);
 }

/******************************************************
 * Kaare indeks. 
 *@param k antud kaar
 *@return Kaare <code>t</code> jrjekorranumber selle graafi kaarte vektoris (vi -1).
*/
 public int indeks(Kaar k){
        return kaared.indexOf(k);
 }

/******************************************************
 * Tipu vljundaste. 
 *@param t antud tipp
 *@return Tipust <code>t</code> vljuvate kaarte arv.
*/
 public int aste(Tipp t){
    int s = 0;
    for(Kaar k: kaared)
       if (k.algus() == t) s++;
    return s;
 }

/******************************************************
 * Tipu sisendaste. 
 *@param t antud tipp
 *@return Tippu <code>t</code> sisenevate kaarte arv.
*/
 public int sisendaste(Tipp t){
    int s = 0;
    for(Kaar k: kaared)
       if (k.lpp() == t) s++;
    return s;
 }

/******************************************************
 * Tipu naabrid. 
 *@param t antud tipp
 *@return Tipu <code>t</code> naabertippudest moodustatud jrjend (vektorina).
*/
 public Vector <Tipp> naabrid(Tipp t){
    Vector <Tipp> tulemus = new  Vector <Tipp> ();
    for(Kaar k: kaared)
       if (k.algus() == t) tulemus.add(k.lpp());
    return tulemus;
 }

/******************************************************
 * Tipust vljuvad kaared. 
 *@param t antud tipp
 *@return Tipust <code>t</code> vljuvatest kaartest moodustatud jrjend (vektorina).
*/
 public Vector <Kaar> vljuvad(Tipp t){
    Vector <Kaar> tulemus = new  Vector <Kaar> ();
    for(Kaar k: kaared)
       if (k.algus() == t) tulemus.add(k);
    return tulemus;
 }

/******************************************************
 * Tippu sisenevad kaared. 
 *@param t antud tipp
 *@return Tippu <code>t</code> sisenevatest kaartest moodustatud jrjend (vektorina).
*/
 public Vector <Kaar> sisenevad(Tipp t){
    Vector <Kaar> tulemus = new  Vector <Kaar> ();
    for(Kaar k: kaared)
       if (k.lpp() == t) tulemus.add(k);
    return tulemus;
 }

/******************************************************
 * Tipu eemaldamine.
 * Eemaldatakse antud tipp koos klgnevate (temast vljuvate ja temasse sisenevate) kaartega. 
 *@param t antud tipp
 *@throws IllegalArgumentException kui tipp <code>t</code> ei ole selles graafis.
*/
 public void eemaldada(Tipp t){
    if (indeks(t) == -1)
              throw new IllegalArgumentException();
    // eemaldada klgnevad kaared:
    for(int i = 0; i < kaared.size(); ){
       Kaar k = kaar(i);
       if (k.algus() == t || k.lpp() == t)
             kaared.removeElement(k);
       else
             i++;
    }
    // eemaldada tipp:
    tipud.removeElement(t);
 }

/******************************************************
 * Juhugraafi loomine.
 * 
 *@param n <code>n > 1</code>
 *@param maxAste <code>maxAste</code> >= 0
 *@return  Silmusteta ja multikaarteta juhugraaf, milles tippuda arv <= <code>n</code>, 
 *         tipu vljundaste <= <code>maxAste</code> ja puuduvad isoleeritud tipud 
(va juht <code>maxAste</code> == 0); 
 *         tipumrgendid on hulgast <code>{A, B, C, ...}</code>; kaaremrgendid on hulgast <code>{a, b, c, ...}</code>.
*/
 public static Graaf juhugraaf(int n, int maxAste){
        Graaf g = new Graaf();
         // tipud;
        char c = 'A';
    	for( int i = 1; i <= n; i++ ){
        	// leida punkt p, mis ei ole juba loodud tippudele liiga lhedal:
		java.awt.Point p;
	     L: for(;;){
			p = new java.awt.Point(juhuarv(0, 500), juhuarv(0, 400) );
			java.awt.Rectangle rk = new java.awt.Rectangle(p.x-Mull.LAIUS, p.y-Mull.KRGUS, Mull.LAIUS*2, Mull.KRGUS*2);
			for(Tipp t : g.tipud()){
				java.awt.Point pt = new java.awt.Point(t.x(), t.y());
				if(rk.contains(pt)) continue L;
		        }
		        break;	
        	}
        	Tipp uus = g. lisada(new Tipp("" + (char)(c + i-1)+ "[0;0]") );
	        uus.seadaKoordinaadid(p.x, p.y );
	}
         // kaared:
        int m = juhuarv(0, n+1);
    	for( int i = 1; i <= m; i++ ){
              	Tipp a;
              	Tipp b;
		for(;;){
                    	a = g.tipp(juhuarv(0, n));
                    	b = g.tipp(juhuarv(0, n));
          		if (a == b) continue;                    
		        if (g.naabrid(a).contains(b)) continue;
                	break;
       		}
       		if (g.aste(a) < maxAste)
	                g.lisada(new Kaar(a, b, "" + (char)('a' + i-1)));
    	}
    	if (maxAste > 0){
               // eemaldada isoleeritud tipud
	       for( int i = 0; i <= g.n()-1; i++ ){
                    	Tipp t = g.tipp(i);
          		if (g.aste(t) == 0 && g.sisendaste(t) == 0 ){
                          	g.tipud.removeElement(t);
                          	i--;
             			if (g.n() < 3) break;
          		}
       		}
    	}
        return g;
 }

/******************************************************
 * Snest (tabelist) graafiks. "Anti-toString()". Eeldus: tipu nimi ei sisalda thikuid. 
 * 
 *@param s toString() tulemus
 *@return  Antud snele vastav graaf.
*/
 public static Graaf graaf(String s)
 {
        Graaf g = new Graaf();
        String[] rida = s.trim().split("\n");
        int n = rida.length;
        Tipp[] tipp = new Tipp[n];
    for( int i = 0; i <= n-1; i++ )
    {
              tipp[i] = g. lisada(new Tipp());
    }
    for( int i = 0; i <= n-1; i++ )
    {
              String ridas = rida[i];
              int k = ridas.indexOf(")");
              ridas = ridas.substring(k+1).trim(); // ilma "nr)"
               // tipu nimi:
              String tipuNimi;
              int kv = ridas.indexOf(">");
       if (kv == -1)
       {
                    tipuNimi = ridas;
                    tipp[i].seadaNimi(tipuNimi);
       }
       else
       {
          if (ridas.length() == 0)
          {
                          tipuNimi = "";
          }
          else
          {
                          int ktyh = ridas.indexOf(" ");
                          tipuNimi = ridas.substring(0, ktyh);
                          ridas = ridas.substring(ktyh+1).trim(); // ilma tipu nimeta
          }
                    tipp[i].seadaNimi(tipuNimi);
                          String[] kaars = ridas.split(">");
                          String kaaremrgend = kaars[0].substring(0, kaars[0].length()-2).trim();
             for( int j = 1; j <= kaars.length-1; j++ )
             {
                                 // kaaremrgend on eelmisest (i-1) olemas
                                String osa = kaars[j].trim();
                                 // osa: "nr mrgend --" vi
                                 // "nr --" vi
                                 // "nr"
                if (osa.endsWith("--"))
                {
                                      osa = osa.substring(0, osa.length()-2).trim(); // "--" lpust ra
                }
                                int ktyh = osa.indexOf(" ");
                                String kuhu;
                if (ktyh == -1)
                {
                                      kuhu = osa;
                }
                else
                {
                                      kuhu = osa.substring(0, ktyh);
                }
                                Tipp lpptipp = tipp[Integer.parseInt(kuhu)-1];
                                g.lisada(new Kaar(tipp[i], lpptipp, kaaremrgend));
                                
                                kaaremrgend = osa.substring(kuhu.length()).trim();
             }
       }
    }
        return g;
 }

/******************************************************
 * Selle graafi snekuju (tabel). 
*/
 public String toString()
 {
        String s = "\n";
        int n = tipud.size();
        int m = kaared.size();
    for( int i = 0; i <= n-1; i++ )
    {
              Tipp ti = tipud.elementAt(i);
              s += (i+1) + ") " + ti.nimi();
       for( int j = 0; j <= m-1; j++ )
       {
                    Kaar kj = kaared.elementAt(j);
          if (kj.algus() == ti)
          {
                          s += " " + kj.nimi() + " --> " + (tipud.indexOf(kj.lpp()) + 1);
          }
       }
              s += "\n";
    }
        return s;
 }

/******************************************************
 * Selle graafi vljastamine faili snekujul tabelina.
 *@param failinimi faili tisnimi vi lihtnimi
*/
 public void  vljastadaTabelina(String failinimi)
 {
    try
    {
              String kuhu = GraafiJoonistaja.tisnimeks(failinimi); // vljundfaili nimi
              PrintWriter  vfail = writerTo(kuhu); // vljundfaili kirjutaja
              
              String rida;
              int n = n();
              int m = m();
       for( int i = 0; i <= n-1; i++ )
       {
                    Tipp ti = tipud.elementAt(i);
                    String s = (i+1) + ") " + ti.nimi();
          for(Kaar kj: kaared)
          {
             if (kj.algus() == ti)
             {
                                s += " " + kj.nimi() + " --> " + (tipud.indexOf(kj.lpp()) + 1);
             }
          }
                    vfail.println(s); // rea kirjutamine vljundfaili
       }
               // lpetamine
              vfail.close();
    }
    catch (IOException e)
    {
              System.err.println(e.getMessage());
    }
 }


/******************************************************
 * Selle graafi sisestamine failist. Graaf failis paikneb snekujul tabelina.
 * 
 *@param failinimi faili tisnimi vi lihtnimi
 *@return  Failist sisestatud  snekujul tabelile vastav graaf.
*/
 public static Graaf sisestadaTabelist(String failinimi)
 {
         // Tulemus: graaf toString() tulemusest (failis failinimi)
         // Eeldus: tipu nimi ei sisalda thikuid
        Graaf g = new Graaf();
        String s = "";
    try
    {
              String kust = GraafiJoonistaja.tisnimeks(failinimi); // sisendfaili nimi
              BufferedReader  sfail = readerFrom(kust); // sisendfailst lugeja
              String rida = "";
       while((rida = sfail.readLine()) != null)
       {
                    s += rida + "\n";
       }
               // lpetamine
              sfail.close();
    }
    catch (IOException e)
    {
              System.err.println(e.getMessage());
              return null;
    }
        return graaf(s);
 }

/******************************************************
 * Juhutisarv. 
 *@param a <code>a>=0</code>
 *@param b <code>b>=0</code> 
 *@return Juhuarv poolligult <code>[a; b)</code> (kui <code>a > <code>b</code>, 
siis poolligult <code>[b; a)</code>).
*/
 static int juhuarv(int a, int b)
 {
         // Antud:  ja b>=0
         // Tulemus: tagastatakse juhuarv poolligult [a; b)
         // (kui a > b, siis poolligult [b; a))
        return (int)(a + Math.random()*(b - a));
 }
 private static  BufferedReader readerFrom(String s) throws FileNotFoundException{// avada tekstiline sisendfail
    return new BufferedReader(new FileReader(s));
 }
 private static PrintWriter writerTo(String s) throws IOException{// avada tekstiline vljundfail
    return new PrintWriter(new FileWriter(s), true);
 }




 }
